from flask.ext import wtf
# noinspection PyProtectedMember
from flask.ext.wtf.form import _is_hidden, _Auto
import inflection


class Form(wtf.Form):
    def __init__(self, formdata=_Auto, obj=None, prefix='', csrf_context=None,
                 secret_key=None, csrf_enabled=None, *args, **kwargs):
        self.form_id = inflection.underscore(self.__class__.__name__)
        if not prefix:
            prefix = self.form_id + '_'
        super(Form, self).__init__(formdata=formdata, obj=obj, prefix=prefix,
                                   csrf_context=csrf_context,
                                   secret_key=secret_key,
                                   csrf_enabled=csrf_enabled, *args, **kwargs)

    def fields(self):
        return tuple(f for f in self if not _is_hidden(f))



from logging import StreamHandler, INFO
from flask import request, url_for, redirect
import flask
from flask.ext.debugtoolbar import DebugToolbarExtension


def current_for(**values):
    return url_for(endpoint=request.endpoint, **values)


def redirect_for(endpoint, **values):
    return redirect(url_for(endpoint=endpoint, **values))


class Request(flask.Request):
    @property
    def is_get(self):
        return self.method == 'GET'

    @property
    def is_post(self):
        return self.method == 'POST'


class Glask(flask.Flask):
    def __init__(self, import_name, static_path=None, static_url_path=None,
                 static_folder='static', template_folder='templates',
                 instance_path=None, instance_relative_config=False):
        super(Glask, self).__init__(import_name, static_path, static_url_path,
                                    static_folder, template_folder,
                                    instance_path, instance_relative_config)
        self.jinja_env.globals.update(current_for=current_for)
        self.request_class = Request
        _config_from_object = self.config.from_object

        def config_from_object(object):
            _config_from_object(object)
            self.init_after_config()

        self.config.from_object = config_from_object

    def init_after_config(self):
        if self.debug:
            DebugToolbarExtension(self)
        else:
            self.logger.addHandler(StreamHandler())
            self.logger.setLevel(INFO)



from __future__ import unicode_literals, print_function, absolute_import, \
    division
from contextlib import closing
from threading import Thread
from time import sleep
from urllib2 import urlopen, URLError

import os
from flask import request
from glask import Glask
import pytest
from selenium import webdriver
from werkzeug.serving import run_simple


class WrapMiddleware():
    def __init__(self):
        self.app = None
        self._default_app = Glask(__name__)
        self.port = 7000
        self.index_url = 'http://localhost:%d' % self.port

        # noinspection PyUnusedLocal
        @self._default_app.route('/')
        def index():
            return ''

        # noinspection PyUnusedLocal
        @self._default_app.route('/quit')
        def quit():
            request.environ['werkzeug.server.shutdown']()
            return ''

    def __call__(self, *args, **kwargs):
        return (self.app or self._default_app)(*args, **kwargs)


# noinspection PyUnusedLocal
@pytest.yield_fixture(scope='session')
def wsgi_server(request):
    server = WrapMiddleware()

    def target():
        run_simple(
            hostname='localhost', application=server, port=server.port,
            use_reloader=False,
        )

    thread = Thread(target=target)
    thread.start()
    # wait for server launched
    while True:
        try:
            with closing(urlopen(server.index_url + '/')) as f:
                f.read()
                break
        except URLError, e:
            if e.args[0].errno != 61:
                raise
        sleep(0.1)

    yield server

    with closing(urlopen(server.index_url + '/quit')) as f:
        f.read()
    thread.join()


# noinspection PyUnusedLocal
@pytest.yield_fixture
def live_app(request, wsgi_server, app):
    wsgi_server.app = app
    with app.test_request_context(wsgi_server.index_url):
        yield app
    wsgi_server.app = None


# noinspection PyUnusedLocal
@pytest.yield_fixture
def client(request, app):
    with app.test_request_context('http://localhost/'):
        yield app.test_client()


@pytest.yield_fixture
def browser():
    type = os.environ.get('TEST_BROWSER', 'firefox')
    b = {'firefox': webdriver.Firefox,
         'phantomjs': webdriver.PhantomJS}.get(type)()
    yield b
    b.quit()



from sys import argv

from setuptools import setup

version = '0.0.13'

install_requires = ['Flask>=0.10.1', 'python-dateutil>=2.2',
                    'Flask-DebugToolbar>=0.9.0', 'Flask-WTF>=0.10.2',
                    'inflection', 'selenium']

if 'develop' in argv[1:]:
    install_requires += ['pytest>=2.5.2', 'Flask-SQLAlchemy']

setup(name='Glask', version=version, packages=['glask'], license='BSD',
      description='An extension for flask applications with best practices.',
      url='https://github.com/vamf12/Glask',
      install_requires=install_requires)
